/* "binso"
 * Copyright 2001-2009 Jetro Lauha
 *
 * Utility to convert binary files to static arrays for compiler
 * to embed the files to the executable.
 *
 * Note that the code has been written no later than 2001 (possibly
 * even earlier than that), and hasn't been really "stress tested"
 * to be bug-free. Any bug reports are still welcome, of course.
 *
 * My home page / blog URL:
 *   http://jet.ro
 *
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Id: binso.c 94 2009-11-09 19:56:36Z tonic $
 * $Revision: 94 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>


int main(int argc, char *argv[])
{
    FILE *infp, *outfp;
    int rowc = 0;
    int ch;
    long byteCount = 0;
    
    if (argc < 4)
    {
	printf("Usage:\n");
	printf("%s input output arrayname\n", argv[0]);
        return EXIT_FAILURE;
    }

    infp = fopen(argv[1], "rb");
    if (infp == NULL)
    {
	printf("Input file (%s) doesn't exist.\n", argv[1]);
	return EXIT_FAILURE;
    }

    outfp = fopen(argv[2], "rb");
    if (outfp != NULL)
    {
	fclose(outfp);
	printf("Output file (%s) already exists.\n"
               "Press Y to overwrite or something else to cancel: ", argv[2]);
	fflush(stdout);
	ch = getch();
	if (toupper(ch) != 'Y')
        {
            printf("n\n");
            return EXIT_FAILURE;
        }
        printf("Y\n");
    }

    outfp = fopen(argv[2], "wb");
    fprintf(outfp, "static unsigned char %s[]={\n", argv[3]);
    
    while ((ch = fgetc(infp)) != EOF)
    {
        ++byteCount;
	++rowc;
        fprintf(outfp, "%d,", ch);
	if (rowc > 20)
        {
            fprintf(outfp, "\n");
	    rowc = 0;
	}
    }

    fprintf(outfp, "\n};\n");

    fclose(outfp);
    fclose(infp);

    printf("Done. (%d bytes)\n", byteCount);
    
    return EXIT_SUCCESS;
}
