/* "binsorle"
 * Copyright 2001-2009 Jetro Lauha
 *
 * Utility to convert binary files to static arrays for compiler
 * to embed the files to the executable. This version has simple
 * RLE packing added to reduce size of repetitive data.
 * Check out testrle.c for unpacking example code.
 *
 * Note that the code has been written no later than 2001 (possibly
 * even earlier than that), and hasn't been really "stress tested"
 * to be bug-free. Any bug reports are still welcome, of course.
 *
 * My home page / blog URL:
 *   http://jet.ro
 *
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * $Id: binsorle.c 94 2009-11-09 19:56:36Z tonic $
 * $Revision: 94 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>


#define u8 unsigned char


int main(int argc, char *argv[])
{
    u8 *indata, *outdata;
    FILE *infp, *outfp;
    long insize, outsize;
    long a, b;
    int rowc = 0;
    int ch;
    u8 lastbyte;
    int count;
    
    if (argc < 4)
    {
	printf("Usage:\n");
	printf("%s input output arrayname\n", argv[0]);
        return EXIT_FAILURE;
    }

    infp = fopen(argv[1], "rb");
    if (infp == NULL)
    {
	printf("Input file (%s) doesn't exist.\n", argv[1]);
	return EXIT_FAILURE;
    }

    outfp = fopen(argv[2], "rb");
    if (outfp != NULL)
    {
	printf("Output file (%s) already exists.\n"
               "Press Y to overwrite or something else to cancel: ", argv[2]);
	fflush(stdout);
	ch = getch();
	if (toupper(ch) != 'Y')
        {
            printf("n\n");
            return EXIT_FAILURE;
        }
	fclose(outfp);
        printf("Y\n");
    }

    outfp = fopen(argv[2], "wb");

    // initialization ok, next load the input file to memory

    fseek(infp, 0, SEEK_END);
    insize = ftell(infp);
    fseek(infp, 0, SEEK_SET);

    indata = (u8 *)malloc(insize);
    if (indata == NULL)
    {
        printf("Out of memory (unable to allocate %d bytes)\n", insize);
        return EXIT_FAILURE;
    }
    fread(indata, insize, 1, infp);

    outdata = (u8 *)malloc(insize + 3 + 4);
    outsize = 4;
    memset(outdata, 0, insize + 3 + 4);
    *(unsigned long *)outdata = insize;

    // rle

    count = 0;
    lastbyte = 0;
    for (a = 0; a < insize; ++a)
    {
        if (count > 0)
        {
            if (lastbyte != indata[a] || count > 0xff)
            {
                if (lastbyte == 0)
                {
                    if (count == 1)
                    {
                        outdata[outsize++] = 0;
                        outdata[outsize++] = 0;
                    } else {
                        outdata[outsize++] = 0;
                        outdata[outsize++] = (u8)(count - 1);
                        outdata[outsize++] = 0;
                    }
                } else {
                    if (count > 3)
                    {
                        outdata[outsize++] = 0;
                        outdata[outsize++] = (u8)(count - 1);
                        outdata[outsize++] = lastbyte;
                    } else {
                        for (b = 0; b < count; ++b)
                            outdata[outsize++] = lastbyte;
                    }
                }                
                count = 0;
            }
        }

        lastbyte = indata[a];
        ++count;

        if (outsize >= insize)
        {
            printf("Output size would be larger than input size!\n"
                   "Store data without rle instead. Aborting...\n");
            fclose(infp);
            fclose(outfp);
            _unlink(argv[2]);
            free(indata);
            free(outdata);
            return EXIT_FAILURE;
        }
    }
    
    // kludge in the trailing bytes
    if (count > 0)
    {
        if (lastbyte == 0)
        {
            if (count == 1)
            {
                outdata[outsize++] = 0;
                outdata[outsize++] = 0;
            } else {
                outdata[outsize++] = 0;
                outdata[outsize++] = (u8)(count - 1);
                outdata[outsize++] = 0;
            }
        } else {
            if (count > 3) {
                outdata[outsize++] = 0;
                outdata[outsize++] = (u8)(count - 1);
                outdata[outsize++] = lastbyte;
            } else {
                for (b = 0; b < count; ++b)
                    outdata[outsize++] = lastbyte;
            }
            
        }
        count = 0;
    }

    fprintf(outfp, "static unsigned char %s[]={\n", argv[3]);

    for (a = 0; a < outsize; ++a)
    {
        ch = outdata[a];
	++rowc;
        fprintf(outfp, "%d,", ch);
	if (rowc > 20)
        {
            fprintf(outfp, "\n");
	    rowc = 0;
	}
    }

    fprintf(outfp,"\n};\n");

    free(indata);
    free(outdata);
    
    fclose(outfp);
    fclose(infp);

    printf("Input size: %d bytes\n", insize);
    printf("Output size: %d bytes (%2.2f%% of input size)\n",
           outsize, 100.0f * outsize / insize);
    printf("Rate: %2.2f%%\n", 100.0f * (insize - outsize) / insize);
    
    return EXIT_SUCCESS;
}
